# encoding: ascii
# api: powershell
# title: Enable-PrintHistory
# description: This enables the Microsoft-Windows-PrintService/Operational in the event log. Every time something is printed, details about the print job will be recorded into that event log.
# version: 0.1
# type: script
# author: KevMar
# license: CC0
# function: Enable-PrintHistory
# x-poshcode-id: 5191
# x-archived: 2014-05-26T06:44:20
# x-published: 2014-05-24T04:25:00
#
#
<#
.Synopsis
   Enables logging of print jobs
.DESCRIPTION
   This enables the Microsoft-Windows-PrintService/Operational in the event log. Every time something is printed, details about the print job will be recorded into that event log.
.EXAMPLE
   Enable-PrintHistory

#>
function Enable-PrintHistory
{

    [CmdletBinding()]
    Param
    (
    )

    Begin
    {
    }
    Process
    {
        Write-Verbose "Check for administrator rights"
        if( ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")){
           
            Write-Verbose "Retreiving Microsoft-Windows-PrintService/Operational Object"
            $EventLog = Get-WinEvent -ListLog Microsoft-Windows-PrintService/Operational 
            Write-Verbose ("Current Values: IsEnabled = {0}; MaximumSize = {1}MB; LogMode = {2}" -f $EventLog.IsEnabled,($EventLog.MaximumSizeInBytes / 1MB),$EventLog.LogMode)
              
            Write-Verbose "Enabling Microsoft-Windows-PrintService/Operational event log"
            Write-Verbose "Setting Values: IsEnabled = True; MaximumSizeInBytes=50MB; LogMode = AutoBackup"
            $EventLog | 
                %{$_.IsEnabled = $true; 
                    $_.MaximumSizeInBytes=50MB;
                    $_.LogMode = "AutoBackup"
                    $_.saveChanges()}
        }else{
            Write-Error "This action requires administrator rights to modify the eventlog"
        }

    }
    End
    {
    }
}
