# encoding: ascii
# api: powershell
# title: Get-MX
# description: Returns the priority mail server (SMTP) to send email directly to the SMTP server of a particular domain/email address.  Uses NetCmdlets (get-dns).
# version: 0.1
# type: function
# author: Lance Robinson
# license: CC0
# function: Get-MX
# x-poshcode-id: 6472
# x-archived: 2016-08-14T11:51:08
# x-published: 2016-08-12T16:25:00
#
#
#Returns the priority mail server (SMTP) for a particular email address.
function Get-MX {
param([string] $domain = $( Throw "Query required in the format domain.com or email@domain.com.") )
	
	#rip domain out of full email address if necessary:
	if ($domain.IndexOf("@")) { $domain = $domain.SubString($domain.IndexOf("@")+1) } 
	  	
  #get all the MX records for this domain, sorted by descending preference 
  $mxrecords = @(get-dns $domain -type MX | sort-object -Property PREFERENCE -des)
  
  #verify that there are some records
  if ($mxrecords.Length -eq 0) { Throw "No records found." }
  	
  #The correct record is the one with the lowest preference:
  $mxrecords[0].EXCHANGE
}
